import os

import boto3
import pytest
from moto import mock_ssm

from ssmenv import SSMEnv, ssmenv


@mock_ssm
@pytest.mark.parametrize(
    "ssm_name_prefix,ssm_value,parameter_type",
    [
        ("/service/org.example.microservice", "1", "String"),
        ("/service/ORG.EXAMPLE.MICROSERVICE", "1", "String"),
        ("/service/ORG-EXAMPLE-MICROSERVICE", "1", "String"),
        ("/service/org-EXAMPLE.microservice", "1", "String"),
        ("/service/org.example.microservice", "1", "SecureString"),
        ("/service/ORG.EXAMPLE.MICROSERVICE", "1", "SecureString"),
        ("/service/ORG-EXAMPLE-MICROSERVICE", "1", "SecureString"),
        ("/service/org-EXAMPLE.microservice", "1", "SecureString"),
    ],
)
def test_it_returns_single_parameter(ssm_name_prefix, ssm_value, parameter_type):
    ssm = boto3.client("ssm")
    ssm.put_parameter(
        Name=ssm_name_prefix + "/debug", Value=ssm_value, Type=parameter_type
    )

    ssm_env = SSMEnv((ssm_name_prefix,))
    assert ssm_env["SERVICE_ORG_EXAMPLE_MICROSERVICE_DEBUG"] == "1"

    param = ssm.get_parameter(Name=ssm_name_prefix + "/debug")
    assert param["Parameter"]["Version"] == 1

    ssm.delete_parameter(Name=ssm_name_prefix)


@mock_ssm
def test_it_returns_multiple_parameters_from_different_namespaces():
    ssm = boto3.client("ssm")
    ssm.put_parameter(Name="/service/my-service/debug", Value="1", Type="String")
    ssm.put_parameter(
        Name="/resource/dynamodb/host", Value="https://example.org", Type="String"
    )

    ssm_env = SSMEnv(("/service/my-service", "/resource/dynamodb"))

    assert ssm_env["SERVICE_MY_SERVICE_DEBUG"] == "1"
    assert ssm_env["RESOURCE_DYNAMODB_HOST"] == "https://example.org"


@mock_ssm
def test_it_allows_to_pass_include_argument_as_string():
    ssm = boto3.client("ssm")
    ssm.put_parameter(Name="/service/my-service/debug", Value="1", Type="String")

    ssm_env = SSMEnv("/service/my-service")

    assert ssm_env["SERVICE_MY_SERVICE_DEBUG"] == "1"


@mock_ssm
def test_it_allows_to_iterate_over_all_parameters():
    ssm = boto3.client("ssm")
    ssm.put_parameter(
        Name="/service/my-service-first/first", Value="first-value", Type="String"
    )
    ssm.put_parameter(
        Name="/service/my-service-first/second", Value="second-value", Type="String"
    )
    ssm.put_parameter(
        Name="/service/my-service-second/first", Value="first-value", Type="String"
    )

    ssm_env = SSMEnv(("/service/my-service-first", "/service/my-service-second"))

    assert list(ssm_env.keys()) == [
        "SERVICE_MY_SERVICE_FIRST_FIRST",
        "SERVICE_MY_SERVICE_FIRST_SECOND",
        "SERVICE_MY_SERVICE_SECOND_FIRST",
    ]
    assert list(ssm_env.values()) == ["first-value", "second-value", "first-value"]


@mock_ssm
def test_it_allows_to_populate_os_environ():
    ssm = boto3.client("ssm")
    ssm.put_parameter(
        Name="/service/my-service/first", Value="first-value", Type="String"
    )

    os.environ = {**os.environ, **SSMEnv(("/service/my-service",))}

    assert os.environ["SERVICE_MY_SERVICE_FIRST"] == "first-value"


@mock_ssm
def test_it_removes_prefixes():
    ssm = boto3.client("ssm")
    ssm.put_parameter(
        Name="/service/my-service/first", Value="first-value", Type="String"
    )

    ssm_env = SSMEnv(("/service/my-service",), prefixes=("/service/my-service",))

    assert ssm_env["FIRST"] == "first-value"


def test_it_returns_default_dict_if_no_aws():
    ssm_env = SSMEnv(
        ("/service/my-service",), no_aws_default={"SERVICE_MY_SERVICE_DEBUG": "1"}
    )

    assert ssm_env["SERVICE_MY_SERVICE_DEBUG"] == "1"


@mock_ssm
def test_it_works_as_decorator_for_lambda_function():
    class MockContext:
        pass

    ssm = boto3.client("ssm")
    ssm.put_parameter(
        Name="/service/my-service/first", Value="first-value", Type="String"
    )

    @ssmenv("/service/my-service")
    def handler(event, context):
        assert "SERVICE_MY_SERVICE_FIRST" in context.params
        assert context.params["SERVICE_MY_SERVICE_FIRST"] == "first-value"

    handler({}, MockContext())



import functools
import os
import re
from collections import UserDict

import boto3

__version__ = "1.4.0"


class SSMEnv(UserDict):
    def __init__(self, include, prefixes=None, ssm_client=None, no_aws_default=None):
        self._include = (include,) if type(include) == str else include
        self._prefixes = prefixes
        self._ssm_client = ssm_client
        self._no_aws_default = no_aws_default

        super().__init__(self._load())

    def _load(self):
        if self._no_aws_default and not any(
            (
                os.environ.get("AWS_ACCESS_KEY_ID"),
                os.environ.get("AWS_CONTAINER_CREDENTIALS_RELATIVE_URI"),
            )
        ):
            return self._no_aws_default

        ssm = self._ssm_client or boto3.client("ssm")

        parameters = {}
        for namespace in self._include:
            params = []
            next_token = -1
            while next_token != 0:
                search_params = {
                    "Path": namespace,
                    "WithDecryption": True,
                    "Recursive": True,
                }

                if next_token != -1:
                    search_params["NextToken"] = next_token

                current_set = ssm.get_parameters_by_path(**search_params)
                params += current_set.get("Parameters")
                next_token = current_set.get("NextToken", 0)

            for param in params:
                name = self._normalize_name(self._remove_prefixes(param.get("Name")))
                parameters[name] = param.get("Value")

        return parameters

    def _remove_prefixes(self, name):
        if not self._prefixes:
            return name

        for prefix in self._prefixes:
            if name.startswith(prefix):
                name = name.replace(prefix, "")

        return name

    def _normalize_name(self, name):
        return re.sub(r"\W", "_", name).upper().strip("_")


_lambda_ssmenv = None


def ssmenv(*args, **kwargs):
    def wrapper_wrapper(handler):
        @functools.wraps(handler)
        def wrapper(event, context):
            if not hasattr(context, "params"):
                context.params = {}

            global _lambda_ssmenv
            if not _lambda_ssmenv:
                _lambda_ssmenv = SSMEnv(*args, **kwargs)
            context.params = (
                _lambda_ssmenv if _lambda_ssmenv else SSMEnv(*args, **kwargs)
            )

            return handler(event, context)

        return wrapper

    return wrapper_wrapper



import json
import os
from setuptools import setup, find_packages

from ssmenv import __version__

with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.md')) as f:
    README = f.read()

with open("Pipfile.lock") as fd:
    lock_data = json.load(fd)
    install_requires = [
        package_name + package_data["version"]
        for package_name, package_data in lock_data["default"].items()
    ]
    tests_require = [
        package_name + package_data["version"]
        for package_name, package_data in lock_data["develop"].items()
    ]

setup(
    name='ssmenv',
    version=__version__,
    description='Allows you to read parameters from AWS Parameter Store and operate on results as on dictionary.',
    long_description=README,
    long_description_content_type="text/markdown",
    keywords="aws boto ssm parameter parameter-store lambda environment-variables",
    author="Daniel Ancuta",
    author_email="whisller@gmail.com",
    url="https://github.com/whisller/ssmenv",
    license="MIT",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries"
    ],
    install_requires=install_requires,
    tests_require=tests_require,
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False
)
